@@ -11,7 +11,7 @@
 #define RULE_RC_SYNTAX_ERROR -1
 #define RULE_RC_REJECT_ERROR -2
 
-int _old_apply_rule (char *rule, int rule_len, char in[BLOCK_SIZE], int in_len, char out[BLOCK_SIZE]);
+int _old_apply_rule (char *rule, int rule_len, char in[RP_PASSWORD_SIZE], int in_len, char out[RP_PASSWORD_SIZE]);
 
 int run_rule_engine (const int rule_len, const char *rule_buf);
 
