@@ -7,6 +7,16 @@
 #include "types.h"
 #include "bitops.h"
 
+u32 __add3_S (const u32 a, const u32 b, const u32 c)
+{
+  return a + b + c;
+}
+
+u32 __add3 (const u32 a, const u32 b, const u32 c)
+{
+  return a + b + c;
+}
+
 u32 rotl32 (const u32 a, const u32 n)
 {
   #if defined (_MSC_VER)
@@ -43,6 +53,12 @@ u64 rotr64 (const u64 a, const u64 n)
   #endif
 }
 
+u16 byte_swap_16 (const u16 n)
+{
+  return (n & 0xff00) >> 8
+       | (n & 0x00ff) << 8;
+}
+
 u32 byte_swap_32 (const u32 n)
 {
   #if defined (_MSC_VER)
