@@ -7,6 +7,7 @@
 #include "types.h"
 #include "memory.h"
 #include "cpt.h"
+#include "shared.h"
 
 int cpt_ctx_init (hashcat_ctx_t *hashcat_ctx)
 {
@@ -15,12 +16,13 @@ int cpt_ctx_init (hashcat_ctx_t *hashcat_ctx)
 
   cpt_ctx->enabled = false;
 
-  if (user_options->keyspace    == true) return 0;
-  if (user_options->left        == true) return 0;
-  if (user_options->opencl_info == true) return 0;
-  if (user_options->show        == true) return 0;
-  if (user_options->usage       == true) return 0;
-  if (user_options->version     == true) return 0;
+  if (user_options->example_hashes == true) return 0;
+  if (user_options->keyspace       == true) return 0;
+  if (user_options->left           == true) return 0;
+  if (user_options->opencl_info    == true) return 0;
+  if (user_options->show           == true) return 0;
+  if (user_options->usage          == true) return 0;
+  if (user_options->version        == true) return 0;
 
   cpt_ctx->enabled = true;
 
@@ -28,7 +30,7 @@ int cpt_ctx_init (hashcat_ctx_t *hashcat_ctx)
 
   cpt_ctx->cpt_total = 0;
   cpt_ctx->cpt_pos   = 0;
-  cpt_ctx->cpt_start = time (NULL);
+  cpt_ctx->cpt_start = hc_time (NULL);
 
   return 0;
 }
@@ -54,5 +56,5 @@ void cpt_ctx_reset (hashcat_ctx_t *hashcat_ctx)
 
   cpt_ctx->cpt_total = 0;
   cpt_ctx->cpt_pos   = 0;
-  cpt_ctx->cpt_start = time (NULL);
+  cpt_ctx->cpt_start = hc_time (NULL);
 }
