@@ -18,6 +18,7 @@
 #include "thread.h"
 #include "status.h"
 #include "interface.h"
+#include "shared.h"
 #include "event.h"
 
 #if defined (__MINGW64__) || defined (__MINGW32__)
@@ -97,15 +98,15 @@ static void main_log (hashcat_ctx_t *hashcat_ctx, FILE *fp, const int loglevel)
   switch (loglevel)
   {
     case LOGLEVEL_INFO:                                   break;
-    case LOGLEVEL_WARNING: fwrite ("\033[33m", 5, 1, fp); break;
-    case LOGLEVEL_ERROR:   fwrite ("\033[31m", 5, 1, fp); break;
-    case LOGLEVEL_ADVICE:  fwrite ("\033[33m", 5, 1, fp); break;
+    case LOGLEVEL_WARNING: hc_fwrite ("\033[33m", 5, 1, fp); break;
+    case LOGLEVEL_ERROR:   hc_fwrite ("\033[31m", 5, 1, fp); break;
+    case LOGLEVEL_ADVICE:  hc_fwrite ("\033[33m", 5, 1, fp); break;
   }
   #endif
 
   // finally, print
 
-  fwrite (msg_buf, msg_len, 1, fp);
+  hc_fwrite (msg_buf, msg_len, 1, fp);
 
   // color stuff post
 
@@ -121,9 +122,9 @@ static void main_log (hashcat_ctx_t *hashcat_ctx, FILE *fp, const int loglevel)
   switch (loglevel)
   {
     case LOGLEVEL_INFO:                                  break;
-    case LOGLEVEL_WARNING: fwrite ("\033[0m", 4, 1, fp); break;
-    case LOGLEVEL_ERROR:   fwrite ("\033[0m", 4, 1, fp); break;
-    case LOGLEVEL_ADVICE:  fwrite ("\033[0m", 4, 1, fp); break;
+    case LOGLEVEL_WARNING: hc_fwrite ("\033[0m", 4, 1, fp); break;
+    case LOGLEVEL_ERROR:   hc_fwrite ("\033[0m", 4, 1, fp); break;
+    case LOGLEVEL_ADVICE:  hc_fwrite ("\033[0m", 4, 1, fp); break;
   }
   #endif
 
@@ -131,13 +132,13 @@ static void main_log (hashcat_ctx_t *hashcat_ctx, FILE *fp, const int loglevel)
 
   if (msg_newline == true)
   {
-    fwrite (EOL, strlen (EOL), 1, fp);
+    hc_fwrite (EOL, strlen (EOL), 1, fp);
 
     // on error, add another newline
 
     if (loglevel == LOGLEVEL_ERROR)
     {
-      fwrite (EOL, strlen (EOL), 1, fp);
+      hc_fwrite (EOL, strlen (EOL), 1, fp);
     }
   }
 
@@ -186,7 +187,7 @@ static void main_outerloop_starting (MAYBE_UNUSED hashcat_ctx_t *hashcat_ctx, MA
 
   status_ctx->shutdown_outer = false;
 
-  if ((user_options->keyspace == false) && (user_options->stdout_flag == false) && (user_options->opencl_info == false) && (user_options->speed_only == false))
+  if ((user_options->example_hashes == false) && (user_options->keyspace == false) && (user_options->stdout_flag == false) && (user_options->opencl_info == false) && (user_options->speed_only == false))
   {
     if ((user_options_extra->wordlist_mode == WL_MODE_FILE) || (user_options_extra->wordlist_mode == WL_MODE_MASK))
     {
@@ -249,9 +250,10 @@ static void main_cracker_finished (MAYBE_UNUSED hashcat_ctx_t *hashcat_ctx, MAYB
   const user_options_t       *user_options       = hashcat_ctx->user_options;
   const user_options_extra_t *user_options_extra = hashcat_ctx->user_options_extra;
 
-  if (user_options->keyspace    == true) return;
-  if (user_options->opencl_info == true) return;
-  if (user_options->stdout_flag == true) return;
+  if (user_options->example_hashes  == true) return;
+  if (user_options->keyspace        == true) return;
+  if (user_options->opencl_info     == true) return;
+  if (user_options->stdout_flag     == true) return;
 
   // if we had a prompt, clear it
 
@@ -323,8 +325,8 @@ static void main_cracker_hash_cracked (MAYBE_UNUSED hashcat_ctx_t *hashcat_ctx,
     if (outfile_ctx->filename == NULL) if (user_options->quiet == false) clear_prompt ();
   }
 
-  fwrite (buf, len,          1, stdout);
-  fwrite (EOL, strlen (EOL), 1, stdout);
+  hc_fwrite (buf, len,          1, stdout);
+  hc_fwrite (EOL, strlen (EOL), 1, stdout);
 
   if ((user_options_extra->wordlist_mode == WL_MODE_FILE) || (user_options_extra->wordlist_mode == WL_MODE_MASK))
   {
@@ -369,8 +371,8 @@ static void main_potfile_hash_show (MAYBE_UNUSED hashcat_ctx_t *hashcat_ctx, MAY
 
   if (outfile_ctx->fp != NULL) return; // cracked hash was not written to an outfile
 
-  fwrite (buf, len,          1, stdout);
-  fwrite (EOL, strlen (EOL), 1, stdout);
+  hc_fwrite (buf, len,          1, stdout);
+  hc_fwrite (EOL, strlen (EOL), 1, stdout);
 }
 
 static void main_potfile_hash_left (MAYBE_UNUSED hashcat_ctx_t *hashcat_ctx, MAYBE_UNUSED const void *buf, MAYBE_UNUSED const size_t len)
@@ -379,8 +381,8 @@ static void main_potfile_hash_left (MAYBE_UNUSED hashcat_ctx_t *hashcat_ctx, MAY
 
   if (outfile_ctx->fp != NULL) return; // cracked hash was not written to an outfile
 
-  fwrite (buf, len,          1, stdout);
-  fwrite (EOL, strlen (EOL), 1, stdout);
+  hc_fwrite (buf, len,          1, stdout);
+  hc_fwrite (EOL, strlen (EOL), 1, stdout);
 }
 
 static void main_potfile_num_cracked (MAYBE_UNUSED hashcat_ctx_t *hashcat_ctx, MAYBE_UNUSED const void *buf, MAYBE_UNUSED const size_t len)
@@ -436,7 +438,7 @@ static void main_outerloop_mainscreen (MAYBE_UNUSED hashcat_ctx_t *hashcat_ctx,
     {
       char *hash_type = strhashtype (hashconfig->hash_mode); // not a bug
 
-      event_log_info (hashcat_ctx, "Hashtype: %s", hash_type);
+      event_log_info (hashcat_ctx, "Hashmode: %d - %s", hashconfig->hash_mode, hash_type);
       event_log_info (hashcat_ctx, NULL);
     }
   }
@@ -467,6 +469,35 @@ static void main_outerloop_mainscreen (MAYBE_UNUSED hashcat_ctx_t *hashcat_ctx,
 
   event_log_info (hashcat_ctx, NULL);
 
+  /**
+   * Optimizer constraints
+   */
+
+  event_log_info (hashcat_ctx, "Password length minimum: %u", hashconfig->pw_min);
+  event_log_info (hashcat_ctx, "Password length maximum: %u", hashconfig->pw_max);
+
+  if (hashconfig->is_salted == true)
+  {
+    if (hashconfig->opti_type & OPTI_TYPE_RAW_HASH)
+    {
+      event_log_info (hashcat_ctx, "Salt length minimum: %u", hashconfig->salt_min);
+      event_log_info (hashcat_ctx, "Salt length maximum: %u", hashconfig->salt_max);
+    }
+  }
+
+  event_log_info (hashcat_ctx, NULL);
+
+  if ((hashconfig->opti_type & OPTI_TYPE_OPTIMIZED_KERNEL) == 0)
+  {
+    if (hashconfig->has_optimized_kernel == true)
+    {
+      event_log_advice (hashcat_ctx, "ATTENTION! Pure (unoptimized) OpenCL kernels selected.");
+      event_log_advice (hashcat_ctx, "This enables cracking passwords and salts > length 32 but for the price of drastical reduced performance.");
+      event_log_advice (hashcat_ctx, "If you want to switch to optimized OpenCL kernels, append -O to your commandline.");
+      event_log_advice (hashcat_ctx, NULL);
+    }
+  }
+
   /**
    * Watchdog and Temperature balance
    */
@@ -496,9 +527,6 @@ static void main_outerloop_mainscreen (MAYBE_UNUSED hashcat_ctx_t *hashcat_ctx,
 
   event_log_info (hashcat_ctx, NULL);
 
-  #if defined (DEBUG)
-  if (user_options->benchmark == true) event_log_info (hashcat_ctx, "Hashmode: %d", hashconfig->hash_mode);
-  #endif
 }
 
 static void main_opencl_session_pre (MAYBE_UNUSED hashcat_ctx_t *hashcat_ctx, MAYBE_UNUSED const void *buf, MAYBE_UNUSED const size_t len)
@@ -519,34 +547,6 @@ static void main_opencl_session_post (MAYBE_UNUSED hashcat_ctx_t *hashcat_ctx, M
   event_log_info_nn (hashcat_ctx, "Initialized device kernels and memory...");
 }
 
-static void main_weak_hash_pre (MAYBE_UNUSED hashcat_ctx_t *hashcat_ctx, MAYBE_UNUSED const void *buf, MAYBE_UNUSED const size_t len)
-{
-  const user_options_t *user_options = hashcat_ctx->user_options;
-
-  if (user_options->quiet == true) return;
-
-  event_log_info_nn (hashcat_ctx, "Checking for weak hashes...");
-}
-
-static void main_weak_hash_post (MAYBE_UNUSED hashcat_ctx_t *hashcat_ctx, MAYBE_UNUSED const void *buf, MAYBE_UNUSED const size_t len)
-{
-  const user_options_t *user_options = hashcat_ctx->user_options;
-
-  if (user_options->quiet == true) return;
-
-  event_log_info_nn (hashcat_ctx, "Checked for weak hashes...");
-}
-
-static void main_weak_hash_all_cracked (MAYBE_UNUSED hashcat_ctx_t *hashcat_ctx, MAYBE_UNUSED const void *buf, MAYBE_UNUSED const size_t len)
-{
-  const user_options_t *user_options = hashcat_ctx->user_options;
-
-  if (user_options->quiet == true) return;
-
-  event_log_info (hashcat_ctx, "INFO: All hashes found during weak hashes check! Use --show to display them.");
-  event_log_info (hashcat_ctx, NULL);
-}
-
 static void main_bitmap_init_pre (MAYBE_UNUSED hashcat_ctx_t *hashcat_ctx, MAYBE_UNUSED const void *buf, MAYBE_UNUSED const size_t len)
 {
   const user_options_t *user_options = hashcat_ctx->user_options;
@@ -648,6 +648,7 @@ static void main_monitor_throttle3 (MAYBE_UNUSED hashcat_ctx_t *hashcat_ctx, MAY
 
 static void main_monitor_performance_hint (MAYBE_UNUSED hashcat_ctx_t *hashcat_ctx, MAYBE_UNUSED const void *buf, MAYBE_UNUSED const size_t len)
 {
+  const hashconfig_t         *hashconfig         = hashcat_ctx->hashconfig;
   const user_options_t       *user_options       = hashcat_ctx->user_options;
   const user_options_extra_t *user_options_extra = hashcat_ctx->user_options_extra;
 
@@ -658,23 +659,34 @@ static void main_monitor_performance_hint (MAYBE_UNUSED hashcat_ctx_t *hashcat_c
     clear_prompt ();
   }
 
+  event_log_advice (hashcat_ctx, "Cracking performance lower than expected?");
+  event_log_advice (hashcat_ctx, NULL);
+
+  if ((hashconfig->opti_type & OPTI_TYPE_OPTIMIZED_KERNEL) == 0)
+  {
+    if (hashconfig->has_optimized_kernel == true)
+    {
+      event_log_advice (hashcat_ctx, "* Append -O to the commandline.");
+      event_log_advice (hashcat_ctx, "  This lowers the maximum supported password- and salt-length (typically down to 32).");
+      event_log_advice (hashcat_ctx, NULL);
+    }
+  }
+
   if (user_options->workload_profile < 3)
   {
-    event_log_advice (hashcat_ctx, "Cracking performance lower than expected? Append -w 3 to the commandline.");
-    event_log_advice (hashcat_ctx, NULL);
-  }
-  else
-  {
-    event_log_advice (hashcat_ctx, "Cracking performance lower than expected?");
-    event_log_advice (hashcat_ctx, NULL);
-    event_log_advice (hashcat_ctx, "* Update your OpenCL runtime / driver the right way:");
-    event_log_advice (hashcat_ctx, "  https://hashcat.net/faq/wrongdriver");
-    event_log_advice (hashcat_ctx, NULL);
-    event_log_advice (hashcat_ctx, "* Create more work items to make use of your parallelization power:");
-    event_log_advice (hashcat_ctx, "  https://hashcat.net/faq/morework");
+    event_log_advice (hashcat_ctx, "* Append -w 3 to the commandline.");
+    event_log_advice (hashcat_ctx, "  This can cause your screen to lag.");
     event_log_advice (hashcat_ctx, NULL);
   }
 
+  event_log_advice (hashcat_ctx, "* Update your OpenCL runtime / driver the right way:");
+  event_log_advice (hashcat_ctx, "  https://hashcat.net/faq/wrongdriver");
+  event_log_advice (hashcat_ctx, NULL);
+  event_log_advice (hashcat_ctx, "* Create more work items to make use of your parallelization power:");
+  event_log_advice (hashcat_ctx, "  https://hashcat.net/faq/morework");
+  event_log_advice (hashcat_ctx, NULL);
+
+
   if ((user_options_extra->wordlist_mode == WL_MODE_FILE) || (user_options_extra->wordlist_mode == WL_MODE_MASK))
   {
     send_prompt ();
@@ -785,21 +797,12 @@ static void main_wordlist_cache_generate (MAYBE_UNUSED hashcat_ctx_t *hashcat_ct
   {
     char *runtime = (char *) malloc (HCBUFSIZ_TINY);
 
-    #if defined (_WIN)
-    __time64_t runtime_sec = cache_generate->runtime;
-    #else
-    time_t runtime_sec = cache_generate->runtime;
-    #endif
+    const hc_time_t runtime_sec = cache_generate->runtime;
 
     struct tm *tmp;
+    struct tm  tm;
 
-    #if defined (_WIN)
-    tmp = _gmtime64 (&runtime_sec);
-    #else
-    struct tm tm;
-
-    tmp = gmtime_r (&runtime_sec, &tm);
-    #endif
+    tmp = hc_gmtime (&runtime_sec, &tm);
 
     format_timer_display (tmp, runtime, HCBUFSIZ_TINY);
 
@@ -954,9 +957,6 @@ static void event (const u32 id, hashcat_ctx_t *hashcat_ctx, const void *buf, co
     case EVENT_POTFILE_REMOVE_PARSE_POST: main_potfile_remove_parse_post (hashcat_ctx, buf, len); break;
     case EVENT_POTFILE_REMOVE_PARSE_PRE:  main_potfile_remove_parse_pre  (hashcat_ctx, buf, len); break;
     case EVENT_SET_KERNEL_POWER_FINAL:    main_set_kernel_power_final    (hashcat_ctx, buf, len); break;
-    case EVENT_WEAK_HASH_POST:            main_weak_hash_post            (hashcat_ctx, buf, len); break;
-    case EVENT_WEAK_HASH_PRE:             main_weak_hash_pre             (hashcat_ctx, buf, len); break;
-    case EVENT_WEAK_HASH_ALL_CRACKED:     main_weak_hash_all_cracked     (hashcat_ctx, buf, len); break;
     case EVENT_WORDLIST_CACHE_GENERATE:   main_wordlist_cache_generate   (hashcat_ctx, buf, len); break;
     case EVENT_WORDLIST_CACHE_HIT:        main_wordlist_cache_hit        (hashcat_ctx, buf, len); break;
   }
@@ -968,7 +968,7 @@ int main (int argc, char **argv)
 
   setup_console ();
 
-  const time_t proc_start = time (NULL);
+  const hc_time_t proc_start = hc_time (NULL);
 
   // hashcat main context
 
@@ -1025,6 +1025,13 @@ int main (int argc, char **argv)
     return 0;
   }
 
+  if (user_options->example_hashes == true)
+  {
+    example_hashes (hashcat_ctx);
+
+    return 0;
+  }
+
   // init a hashcat session; this initializes opencl devices, hwmon, etc
 
   welcome_screen (hashcat_ctx, VERSION_TAG);
@@ -1049,6 +1056,8 @@ int main (int argc, char **argv)
 
       opencl_info_compact (hashcat_ctx);
 
+      user_options_info (hashcat_ctx);
+
       rc_final = hashcat_session_execute (hashcat_ctx);
     }
   }
@@ -1059,7 +1068,7 @@ int main (int argc, char **argv)
 
   // finished with hashcat, clean up
 
-  const time_t proc_stop = time (NULL);
+  const hc_time_t proc_stop = hc_time (NULL);
 
   goodbye_screen (hashcat_ctx, proc_start, proc_stop);
 
