@@ -58,7 +58,8 @@ static const char grp_op_chr[] =
 {
   RULE_OP_MANGLE_APPEND,
   RULE_OP_MANGLE_PREPEND,
-  RULE_OP_MANGLE_PURGECHAR
+  RULE_OP_MANGLE_PURGECHAR,
+  RULE_OP_MANGLE_TITLE_SEP
 };
 
 static const char grp_op_chr_chr[] =
@@ -110,33 +111,21 @@ bool class_alpha (const u8 c)
 
 int conv_ctoi (const u8 c)
 {
-  if (class_num (c))
-  {
-    return c - '0';
-  }
-  else if (class_upper (c))
-  {
-    return c - 'A' + 10;
-  }
+  if (class_num (c)) return c - '0';
+  if (class_upper (c)) return c - 'A' + 10;
 
   return -1;
 }
 
 int conv_itoc (const u8 c)
 {
-  if (c < 10)
-  {
-    return c + '0';
-  }
-  else if (c < 37)
-  {
-    return c + 'A' - 10;
-  }
+  if (c < 10) return c + '0';
+  if (c < 37) return c + 'A' - 10;
 
   return -1;
 }
 
-int generate_random_rule (char rule_buf[RP_RULE_BUFSIZ], const u32 rp_gen_func_min, const u32 rp_gen_func_max)
+int generate_random_rule (char rule_buf[RP_RULE_SIZE], const u32 rp_gen_func_min, const u32 rp_gen_func_max)
 {
   u32 rp_gen_num = get_random_num (rp_gen_func_min, rp_gen_func_max);
 
@@ -444,6 +433,11 @@ int cpu_rule_to_kernel_rule (char *rule_buf, u32 rule_len, kernel_rule_t *rule)
         SET_NAME (rule, rule_buf[rule_pos]);
         break;
 
+      case RULE_OP_MANGLE_TITLE_SEP:
+        SET_NAME (rule, rule_buf[rule_pos]);
+        SET_P0   (rule, rule_buf[rule_pos]);
+        break;
+
       default:
         return -1;
     }
@@ -662,7 +656,13 @@ int kernel_rule_to_cpu_rule (char *rule_buf, kernel_rule_t *rule)
         rule_buf[rule_pos] = rule_cmd;
         break;
 
+      case RULE_OP_MANGLE_TITLE_SEP:
+        rule_buf[rule_pos] = rule_cmd;
+        GET_P0 (rule);
+        break;
+
       case 0:
+        if (rule_pos == 0) return -1;
         return rule_pos - 1;
 
       default:
@@ -670,12 +670,7 @@ int kernel_rule_to_cpu_rule (char *rule_buf, kernel_rule_t *rule)
     }
   }
 
-  if (rule_cnt > 0)
-  {
-    return rule_pos;
-  }
-
-  return -1;
+  return rule_pos;
 }
 
 bool kernel_rules_has_noop (const kernel_rule_t *kernel_rules_buf, const u32 kernel_rules_cnt)
@@ -724,16 +719,18 @@ int kernel_rules_load (hashcat_ctx_t *hashcat_ctx, kernel_rule_t **out_buf, u32
 
     char *rp_file = user_options->rp_files[i];
 
-    char in[BLOCK_SIZE]  = { 0 };
-    char out[BLOCK_SIZE] = { 0 };
-
     FILE *fp = NULL;
 
     u32 rule_line = 0;
 
     if ((fp = fopen (rp_file, "rb")) == NULL)
     {
-      event_log_error (hashcat_ctx, "%s: %m", rp_file);
+      event_log_error (hashcat_ctx, "%s: %s", rp_file, strerror (errno));
+
+      hcfree (all_kernel_rules_cnt);
+      hcfree (all_kernel_rules_buf);
+
+      hcfree (rule_buf);
 
       return -1;
     }
@@ -755,8 +752,11 @@ int kernel_rules_load (hashcat_ctx_t *hashcat_ctx, kernel_rule_t **out_buf, u32
         kernel_rules_avail += INCR_RULES;
       }
 
-      memset (in,  0, BLOCK_SIZE);
-      memset (out, 0, BLOCK_SIZE);
+      char in[RP_PASSWORD_SIZE];
+      char out[RP_PASSWORD_SIZE];
+
+      memset (in,  0, sizeof (in));
+      memset (out, 0, sizeof (out));
 
       int result = _old_apply_rule (rule_buf, rule_len, in, 1, out);
 
@@ -823,7 +823,7 @@ int kernel_rules_load (hashcat_ctx_t *hashcat_ctx, kernel_rule_t **out_buf, u32
       {
         if (out_pos == RULES_MAX - 1)
         {
-          // event_log_warning (hashcat_ctx, "Truncating chaining of rule %d and rule %d as maximum number of function calls per rule exceeded", i, in_off);
+          // event_log_warning (hashcat_ctx, "Truncated chaining of rule %d and rule %d - maximum functions per rule exceeded.", i, in_off);
 
           break;
         }
@@ -835,16 +835,18 @@ int kernel_rules_load (hashcat_ctx_t *hashcat_ctx, kernel_rule_t **out_buf, u32
 
   hcfree (repeats);
 
+  hcfree (all_kernel_rules_cnt);
+  hcfree (all_kernel_rules_buf);
+
   if (kernel_rules_cnt == 0)
   {
-    event_log_error (hashcat_ctx, "No valid rules left");
+    event_log_error (hashcat_ctx, "No valid rules left.");
+
+    hcfree (kernel_rules_buf);
 
     return -1;
   }
 
-  hcfree (all_kernel_rules_cnt);
-  hcfree (all_kernel_rules_buf);
-
   *out_cnt = kernel_rules_cnt;
   *out_buf = kernel_rules_buf;
 
@@ -858,13 +860,13 @@ int kernel_rules_generate (hashcat_ctx_t *hashcat_ctx, kernel_rule_t **out_buf,
   u32            kernel_rules_cnt = 0;
   kernel_rule_t *kernel_rules_buf = hccalloc (user_options->rp_gen, sizeof (kernel_rule_t));
 
-  char *rule_buf = (char *) hcmalloc (RP_RULE_BUFSIZ);
+  char *rule_buf = (char *) hcmalloc (RP_RULE_SIZE);
 
   for (kernel_rules_cnt = 0; kernel_rules_cnt < user_options->rp_gen; kernel_rules_cnt++)
   {
-    memset (rule_buf, 0, RP_RULE_BUFSIZ);
+    memset (rule_buf, 0, RP_RULE_SIZE);
 
-    int rule_len = (int) generate_random_rule (rule_buf, user_options->rp_gen_func_min, user_options->rp_gen_func_max);
+    int rule_len = generate_random_rule (rule_buf, user_options->rp_gen_func_min, user_options->rp_gen_func_max);
 
     if (cpu_rule_to_kernel_rule (rule_buf, rule_len, &kernel_rules_buf[kernel_rules_cnt]) == -1) continue;
   }
