@@ -11,15 +11,14 @@
 #define NEXT_RULEPOS(rp)      if (++(rp) == rule_len) return (RULE_RC_SYNTAX_ERROR)
 #define NEXT_RPTOI(r,rp,up)   if (((up) = conv_pos ((r)[(rp)], pos_mem)) == -1) return (RULE_RC_SYNTAX_ERROR)
 
-static int conv_pos (const u8 c, const int pos_mem) {
+static int conv_pos (const u8 c, const int pos_mem)
+{
   if (c == RULE_LAST_REJECTED_SAVED_POS)
   {
     return pos_mem;
   }
-  else
-  {
-    return conv_ctoi (c);
-  }
+
+  return conv_ctoi (c);
 }
 
 static void MANGLE_TOGGLE_AT (char *arr, const int pos)
@@ -44,7 +43,7 @@ static void MANGLE_SWITCH (char *arr, const int l, const int r)
   arr[l] = c;
 }
 
-static int mangle_lrest (char arr[BLOCK_SIZE], int arr_len)
+static int mangle_lrest (char arr[RP_PASSWORD_SIZE], int arr_len)
 {
   int pos;
 
@@ -53,7 +52,7 @@ static int mangle_lrest (char arr[BLOCK_SIZE], int arr_len)
   return (arr_len);
 }
 
-static int mangle_urest (char arr[BLOCK_SIZE], int arr_len)
+static int mangle_urest (char arr[RP_PASSWORD_SIZE], int arr_len)
 {
   int pos;
 
@@ -62,7 +61,7 @@ static int mangle_urest (char arr[BLOCK_SIZE], int arr_len)
   return (arr_len);
 }
 
-static int mangle_trest (char arr[BLOCK_SIZE], int arr_len)
+static int mangle_trest (char arr[RP_PASSWORD_SIZE], int arr_len)
 {
   int pos;
 
@@ -71,7 +70,7 @@ static int mangle_trest (char arr[BLOCK_SIZE], int arr_len)
   return (arr_len);
 }
 
-static int mangle_reverse (char arr[BLOCK_SIZE], int arr_len)
+static int mangle_reverse (char arr[RP_PASSWORD_SIZE], int arr_len)
 {
   int l;
 
@@ -87,18 +86,18 @@ static int mangle_reverse (char arr[BLOCK_SIZE], int arr_len)
   return (arr_len);
 }
 
-static int mangle_double (char arr[BLOCK_SIZE], int arr_len)
+static int mangle_double (char arr[RP_PASSWORD_SIZE], int arr_len)
 {
-  if ((arr_len * 2) >= BLOCK_SIZE) return (arr_len);
+  if ((arr_len * 2) >= RP_PASSWORD_SIZE) return (arr_len);
 
   memcpy (&arr[arr_len], arr, (size_t) arr_len);
 
   return (arr_len * 2);
 }
 
-static int mangle_double_times (char arr[BLOCK_SIZE], int arr_len, int times)
+static int mangle_double_times (char arr[RP_PASSWORD_SIZE], int arr_len, int times)
 {
-  if (((arr_len * times) + arr_len) >= BLOCK_SIZE) return (arr_len);
+  if (((arr_len * times) + arr_len) >= RP_PASSWORD_SIZE) return (arr_len);
 
   int orig_len = arr_len;
 
@@ -114,9 +113,9 @@ static int mangle_double_times (char arr[BLOCK_SIZE], int arr_len, int times)
   return (arr_len);
 }
 
-static int mangle_reflect (char arr[BLOCK_SIZE], int arr_len)
+static int mangle_reflect (char arr[RP_PASSWORD_SIZE], int arr_len)
 {
-  if ((arr_len * 2) >= BLOCK_SIZE) return (arr_len);
+  if ((arr_len * 2) >= RP_PASSWORD_SIZE) return (arr_len);
 
   mangle_double (arr, arr_len);
 
@@ -125,7 +124,7 @@ static int mangle_reflect (char arr[BLOCK_SIZE], int arr_len)
   return (arr_len * 2);
 }
 
-static int mangle_rotate_left (char arr[BLOCK_SIZE], int arr_len)
+static int mangle_rotate_left (char arr[RP_PASSWORD_SIZE], int arr_len)
 {
   int l;
   int r;
@@ -138,7 +137,7 @@ static int mangle_rotate_left (char arr[BLOCK_SIZE], int arr_len)
   return (arr_len);
 }
 
-static int mangle_rotate_right (char arr[BLOCK_SIZE], int arr_len)
+static int mangle_rotate_right (char arr[RP_PASSWORD_SIZE], int arr_len)
 {
   int l;
   int r;
@@ -151,18 +150,18 @@ static int mangle_rotate_right (char arr[BLOCK_SIZE], int arr_len)
   return (arr_len);
 }
 
-static int mangle_append (char arr[BLOCK_SIZE], int arr_len, char c)
+static int mangle_append (char arr[RP_PASSWORD_SIZE], int arr_len, char c)
 {
-  if ((arr_len + 1) >= BLOCK_SIZE) return (arr_len);
+  if ((arr_len + 1) >= RP_PASSWORD_SIZE) return (arr_len);
 
   arr[arr_len] = c;
 
   return (arr_len + 1);
 }
 
-static int mangle_prepend (char arr[BLOCK_SIZE], int arr_len, char c)
+static int mangle_prepend (char arr[RP_PASSWORD_SIZE], int arr_len, char c)
 {
-  if ((arr_len + 1) >= BLOCK_SIZE) return (arr_len);
+  if ((arr_len + 1) >= RP_PASSWORD_SIZE) return (arr_len);
 
   int arr_pos;
 
@@ -176,7 +175,7 @@ static int mangle_prepend (char arr[BLOCK_SIZE], int arr_len, char c)
   return (arr_len + 1);
 }
 
-static int mangle_delete_at (char arr[BLOCK_SIZE], int arr_len, int upos)
+static int mangle_delete_at (char arr[RP_PASSWORD_SIZE], int arr_len, int upos)
 {
   if (upos >= arr_len) return (arr_len);
 
@@ -190,7 +189,7 @@ static int mangle_delete_at (char arr[BLOCK_SIZE], int arr_len, int upos)
   return (arr_len - 1);
 }
 
-static int mangle_extract (char arr[BLOCK_SIZE], int arr_len, int upos, int ulen)
+static int mangle_extract (char arr[RP_PASSWORD_SIZE], int arr_len, int upos, int ulen)
 {
   if (upos >= arr_len) return (arr_len);
 
@@ -206,7 +205,7 @@ static int mangle_extract (char arr[BLOCK_SIZE], int arr_len, int upos, int ulen
   return (ulen);
 }
 
-static int mangle_omit (char arr[BLOCK_SIZE], int arr_len, int upos, int ulen)
+static int mangle_omit (char arr[RP_PASSWORD_SIZE], int arr_len, int upos, int ulen)
 {
   if (upos >= arr_len) return (arr_len);
 
@@ -222,11 +221,11 @@ static int mangle_omit (char arr[BLOCK_SIZE], int arr_len, int upos, int ulen)
   return (arr_len - ulen);
 }
 
-static int mangle_insert (char arr[BLOCK_SIZE], int arr_len, int upos, char c)
+static int mangle_insert (char arr[RP_PASSWORD_SIZE], int arr_len, int upos, char c)
 {
   if (upos > arr_len) return (arr_len);
 
-  if ((arr_len + 1) >= BLOCK_SIZE) return (arr_len);
+  if ((arr_len + 1) >= RP_PASSWORD_SIZE) return (arr_len);
 
   int arr_pos;
 
@@ -240,9 +239,9 @@ static int mangle_insert (char arr[BLOCK_SIZE], int arr_len, int upos, char c)
   return (arr_len + 1);
 }
 
-static int mangle_insert_multi (char arr[BLOCK_SIZE], int arr_len, int arr_pos, char arr2[BLOCK_SIZE], int arr2_len, int arr2_pos, int arr2_cpy)
+static int mangle_insert_multi (char arr[RP_PASSWORD_SIZE], int arr_len, int arr_pos, char arr2[RP_PASSWORD_SIZE], int arr2_len, int arr2_pos, int arr2_cpy)
 {
-  if ((arr_len + arr2_cpy) > BLOCK_SIZE) return (RULE_RC_REJECT_ERROR);
+  if ((arr_len + arr2_cpy) > RP_PASSWORD_SIZE) return (RULE_RC_REJECT_ERROR);
 
   if (arr_pos > arr_len) return (RULE_RC_REJECT_ERROR);
 
@@ -252,16 +251,16 @@ static int mangle_insert_multi (char arr[BLOCK_SIZE], int arr_len, int arr_pos,
 
   if (arr2_cpy < 1) return (RULE_RC_SYNTAX_ERROR);
 
-  memcpy (arr2, arr2 + arr2_pos, arr2_len - arr2_pos);
+  memmove (arr2, arr2 + arr2_pos, arr2_len - arr2_pos);
 
-  memcpy (arr2 + arr2_cpy, arr + arr_pos, arr_len - arr_pos);
+  memcpy  (arr2 + arr2_cpy, arr + arr_pos, arr_len - arr_pos);
 
-  memcpy (arr + arr_pos, arr2, arr_len - arr_pos + arr2_cpy);
+  memcpy  (arr + arr_pos, arr2, arr_len - arr_pos + arr2_cpy);
 
   return (arr_len + arr2_cpy);
 }
 
-static int mangle_overstrike (char arr[BLOCK_SIZE], int arr_len, int upos, char c)
+static int mangle_overstrike (char arr[RP_PASSWORD_SIZE], int arr_len, int upos, char c)
 {
   if (upos >= arr_len) return (arr_len);
 
@@ -270,7 +269,7 @@ static int mangle_overstrike (char arr[BLOCK_SIZE], int arr_len, int upos, char
   return (arr_len);
 }
 
-static int mangle_truncate_at (char arr[BLOCK_SIZE], int arr_len, int upos)
+static int mangle_truncate_at (char arr[RP_PASSWORD_SIZE], int arr_len, int upos)
 {
   if (upos >= arr_len) return (arr_len);
 
@@ -279,7 +278,7 @@ static int mangle_truncate_at (char arr[BLOCK_SIZE], int arr_len, int upos)
   return (upos);
 }
 
-static int mangle_replace (char arr[BLOCK_SIZE], int arr_len, char oldc, char newc)
+static int mangle_replace (char arr[RP_PASSWORD_SIZE], int arr_len, char oldc, char newc)
 {
   int arr_pos;
 
@@ -293,7 +292,7 @@ static int mangle_replace (char arr[BLOCK_SIZE], int arr_len, char oldc, char ne
   return (arr_len);
 }
 
-static int mangle_purgechar (char arr[BLOCK_SIZE], int arr_len, char c)
+static int mangle_purgechar (char arr[RP_PASSWORD_SIZE], int arr_len, char c)
 {
   int arr_pos;
 
@@ -311,11 +310,11 @@ static int mangle_purgechar (char arr[BLOCK_SIZE], int arr_len, char c)
   return (ret_len);
 }
 
-static int mangle_dupeblock_prepend (char arr[BLOCK_SIZE], int arr_len, int ulen)
+static int mangle_dupeblock_prepend (char arr[RP_PASSWORD_SIZE], int arr_len, int ulen)
 {
   if (ulen > arr_len) return (arr_len);
 
-  if ((arr_len + ulen) >= BLOCK_SIZE) return (arr_len);
+  if ((arr_len + ulen) >= RP_PASSWORD_SIZE) return (arr_len);
 
   char cs[100] = { 0 };
 
@@ -333,11 +332,11 @@ static int mangle_dupeblock_prepend (char arr[BLOCK_SIZE], int arr_len, int ulen
   return (arr_len);
 }
 
-static int mangle_dupeblock_append (char arr[BLOCK_SIZE], int arr_len, int ulen)
+static int mangle_dupeblock_append (char arr[RP_PASSWORD_SIZE], int arr_len, int ulen)
 {
   if (ulen > arr_len) return (arr_len);
 
-  if ((arr_len + ulen) >= BLOCK_SIZE) return (arr_len);
+  if ((arr_len + ulen) >= RP_PASSWORD_SIZE) return (arr_len);
 
   int upos = arr_len - ulen;
 
@@ -353,10 +352,10 @@ static int mangle_dupeblock_append (char arr[BLOCK_SIZE], int arr_len, int ulen)
   return (arr_len);
 }
 
-static int mangle_dupechar_at (char arr[BLOCK_SIZE], int arr_len, int upos, int ulen)
+static int mangle_dupechar_at (char arr[RP_PASSWORD_SIZE], int arr_len, int upos, int ulen)
 {
   if ( arr_len         ==  0) return (arr_len);
-  if ((arr_len + ulen) >= BLOCK_SIZE) return (arr_len);
+  if ((arr_len + ulen) >= RP_PASSWORD_SIZE) return (arr_len);
 
   char c = arr[upos];
 
@@ -370,10 +369,10 @@ static int mangle_dupechar_at (char arr[BLOCK_SIZE], int arr_len, int upos, int
   return (arr_len);
 }
 
-static int mangle_dupechar (char arr[BLOCK_SIZE], int arr_len)
+static int mangle_dupechar (char arr[RP_PASSWORD_SIZE], int arr_len)
 {
   if ( arr_len            ==  0) return (arr_len);
-  if ((arr_len + arr_len) >= BLOCK_SIZE) return (arr_len);
+  if ((arr_len + arr_len) >= RP_PASSWORD_SIZE) return (arr_len);
 
   int arr_pos;
 
@@ -389,7 +388,7 @@ static int mangle_dupechar (char arr[BLOCK_SIZE], int arr_len)
   return (arr_len * 2);
 }
 
-static int mangle_switch_at_check (char arr[BLOCK_SIZE], int arr_len, int upos, int upos2)
+static int mangle_switch_at_check (char arr[RP_PASSWORD_SIZE], int arr_len, int upos, int upos2)
 {
   if (upos  >= arr_len) return (arr_len);
   if (upos2 >= arr_len) return (arr_len);
@@ -399,14 +398,14 @@ static int mangle_switch_at_check (char arr[BLOCK_SIZE], int arr_len, int upos,
   return (arr_len);
 }
 
-static int mangle_switch_at (char arr[BLOCK_SIZE], int arr_len, int upos, int upos2)
+static int mangle_switch_at (char arr[RP_PASSWORD_SIZE], int arr_len, int upos, int upos2)
 {
   MANGLE_SWITCH (arr, upos, upos2);
 
   return (arr_len);
 }
 
-static int mangle_chr_shiftl (char arr[BLOCK_SIZE], int arr_len, int upos)
+static int mangle_chr_shiftl (char arr[RP_PASSWORD_SIZE], int arr_len, int upos)
 {
   if (upos >= arr_len) return (arr_len);
 
@@ -415,7 +414,7 @@ static int mangle_chr_shiftl (char arr[BLOCK_SIZE], int arr_len, int upos)
   return (arr_len);
 }
 
-static int mangle_chr_shiftr (char arr[BLOCK_SIZE], int arr_len, int upos)
+static int mangle_chr_shiftr (char arr[RP_PASSWORD_SIZE], int arr_len, int upos)
 {
   if (upos >= arr_len) return (arr_len);
 
@@ -424,7 +423,7 @@ static int mangle_chr_shiftr (char arr[BLOCK_SIZE], int arr_len, int upos)
   return (arr_len);
 }
 
-static int mangle_chr_incr (char arr[BLOCK_SIZE], int arr_len, int upos)
+static int mangle_chr_incr (char arr[RP_PASSWORD_SIZE], int arr_len, int upos)
 {
   if (upos >= arr_len) return (arr_len);
 
@@ -433,7 +432,7 @@ static int mangle_chr_incr (char arr[BLOCK_SIZE], int arr_len, int upos)
   return (arr_len);
 }
 
-static int mangle_chr_decr (char arr[BLOCK_SIZE], int arr_len, int upos)
+static int mangle_chr_decr (char arr[RP_PASSWORD_SIZE], int arr_len, int upos)
 {
   if (upos >= arr_len) return (arr_len);
 
@@ -442,7 +441,7 @@ static int mangle_chr_decr (char arr[BLOCK_SIZE], int arr_len, int upos)
   return (arr_len);
 }
 
-static int mangle_title_sep (char arr[BLOCK_SIZE], int arr_len, char c)
+static int mangle_title_sep (char arr[RP_PASSWORD_SIZE], int arr_len, char c)
 {
   int upper_next = 1;
 
@@ -472,16 +471,17 @@ static int mangle_title_sep (char arr[BLOCK_SIZE], int arr_len, char c)
   return (arr_len);
 }
 
-int _old_apply_rule (char *rule, int rule_len, char in[BLOCK_SIZE], int in_len, char out[BLOCK_SIZE])
+int _old_apply_rule (char *rule, int rule_len, char in[RP_PASSWORD_SIZE], int in_len, char out[RP_PASSWORD_SIZE])
 {
-  char mem[BLOCK_SIZE] = { 0 };
+  char mem[RP_PASSWORD_SIZE] = { 0 };
+
   int pos_mem = -1;
 
   if (in == NULL) return (RULE_RC_REJECT_ERROR);
 
   if (out == NULL) return (RULE_RC_REJECT_ERROR);
 
-  if (in_len < 1 || in_len > BLOCK_SIZE) return (RULE_RC_REJECT_ERROR);
+  if (in_len < 0 || in_len > RP_PASSWORD_SIZE) return (RULE_RC_REJECT_ERROR);
 
   if (rule_len < 1) return (RULE_RC_REJECT_ERROR);
 
@@ -736,14 +736,14 @@ int _old_apply_rule (char *rule, int rule_len, char in[BLOCK_SIZE], int in_len,
 
       case RULE_OP_MANGLE_APPEND_MEMORY:
         if (mem_len < 1) return (RULE_RC_REJECT_ERROR);
-        if ((out_len + mem_len) >= BLOCK_SIZE) return (RULE_RC_REJECT_ERROR);
+        if ((out_len + mem_len) >= RP_PASSWORD_SIZE) return (RULE_RC_REJECT_ERROR);
         memcpy (out + out_len, mem, mem_len);
         out_len += mem_len;
         break;
 
       case RULE_OP_MANGLE_PREPEND_MEMORY:
         if (mem_len < 1) return (RULE_RC_REJECT_ERROR);
-        if ((mem_len + out_len) >= BLOCK_SIZE) return (RULE_RC_REJECT_ERROR);
+        if ((mem_len + out_len) >= RP_PASSWORD_SIZE) return (RULE_RC_REJECT_ERROR);
         memcpy (mem + mem_len, out, out_len);
         out_len += mem_len;
         memcpy (out, mem, out_len);
@@ -814,7 +814,8 @@ int _old_apply_rule (char *rule, int rule_len, char in[BLOCK_SIZE], int in_len,
         if ((upos + 1) > out_len) return (RULE_RC_REJECT_ERROR);
         NEXT_RULEPOS (rule_pos);
         int c; int cnt;
-        for (c = 0, cnt = 0; c < out_len && cnt < upos; c++) {
+        for (c = 0, cnt = 0; c < out_len && cnt < upos; c++)
+        {
           if (out[c] == rule[rule_pos])
           {
             cnt++;
@@ -834,21 +835,17 @@ int _old_apply_rule (char *rule, int rule_len, char in[BLOCK_SIZE], int in_len,
     }
   }
 
-  memset (out + out_len, 0, BLOCK_SIZE - out_len);
+  memset (out + out_len, 0, RP_PASSWORD_SIZE - out_len);
 
   return (out_len);
 }
 
 int run_rule_engine (const int rule_len, const char *rule_buf)
 {
-  if (rule_len == 0)
-  {
-    return 0;
-  }
-  else if (rule_len == 1)
-  {
+  if (rule_len == 0) return 0;
+
+  if (rule_len == 1)
     if (rule_buf[0] == RULE_OP_MANGLE_NOOP) return 0;
-  }
 
   return 1;
 }
