@@ -8,15 +8,17 @@
 #include "event.h"
 #include "locking.h"
 #include "rp_kernel_on_cpu.h"
+#include "rp_kernel_on_cpu_optimized.h"
 #include "mpsp.h"
 #include "opencl.h"
+#include "shared.h"
 #include "stdout.h"
 
 static void out_flush (out_t *out)
 {
   if (out->len == 0) return;
 
-  fwrite (out->buf, 1, out->len, out->fp);
+  hc_fwrite (out->buf, 1, out->len, out->fp);
 
   out->len = 0;
 }
@@ -27,22 +29,22 @@ static void out_push (out_t *out, const u8 *pw_buf, const int pw_len)
 
   memcpy (ptr, pw_buf, pw_len);
 
-  #if defined (_POSIX)
-
-  ptr[pw_len] = '\n';
-
-  out->len += pw_len + 1;
-
-  #else
+  #if defined (_WIN)
 
   ptr[pw_len + 0] = '\r';
   ptr[pw_len + 1] = '\n';
 
   out->len += pw_len + 2;
 
+  #else
+
+  ptr[pw_len] = '\n';
+
+  out->len += pw_len + 1;
+
   #endif
 
-  if (out->len >= BUFSIZ - 100)
+  if (out->len >= HCBUFSIZ_TINY - 300)
   {
     out_flush (out);
   }
@@ -69,7 +71,7 @@ int process_stdout (hashcat_ctx_t *hashcat_ctx, hc_device_param_t *device_param,
 
     if (fp == NULL)
     {
-      event_log_error (hashcat_ctx, "%s: %m", filename);
+      event_log_error (hashcat_ctx, "%s: %s", filename, strerror (errno));
 
       return -1;
     }
@@ -78,7 +80,7 @@ int process_stdout (hashcat_ctx_t *hashcat_ctx, hc_device_param_t *device_param,
     {
       fclose (fp);
 
-      event_log_error (hashcat_ctx, "%s: %m", filename);
+      event_log_error (hashcat_ctx, "%s: %s", filename, strerror (errno));
 
       return -1;
     }
@@ -88,7 +90,7 @@ int process_stdout (hashcat_ctx_t *hashcat_ctx, hc_device_param_t *device_param,
 
   out.len = 0;
 
-  u32 plain_buf[16] = { 0 };
+  u32 plain_buf[64] = { 0 };
 
   u8 *plain_ptr = (u8 *) plain_buf;
 
@@ -111,18 +113,28 @@ int process_stdout (hashcat_ctx_t *hashcat_ctx, hc_device_param_t *device_param,
         return -1;
       }
 
-      const u32 pos = device_param->innerloop_pos;
-
       for (u32 il_pos = 0; il_pos < il_cnt; il_pos++)
       {
-        for (int i = 0; i < 8; i++)
+        const u32 off = device_param->innerloop_pos + il_pos;
+
+        if (hashconfig->opti_type & OPTI_TYPE_OPTIMIZED_KERNEL)
         {
-          plain_buf[i] = pw.i[i];
+          for (int i = 0; i < 8; i++)
+          {
+            plain_buf[i] = pw.i[i];
+          }
+
+          plain_len = apply_rules_optimized (straight_ctx->kernel_rules_buf[off].cmds, &plain_buf[0], &plain_buf[4], pw.pw_len);
         }
+        else
+        {
+          for (int i = 0; i < 64; i++)
+          {
+            plain_buf[i] = pw.i[i];
+          }
 
-        plain_len = pw.pw_len;
-
-        plain_len = apply_rules (straight_ctx->kernel_rules_buf[pos + il_pos].cmds, &plain_buf[0], &plain_buf[4], plain_len);
+          plain_len = apply_rules (straight_ctx->kernel_rules_buf[off].cmds, plain_buf, pw.pw_len);
+        }
 
         if (plain_len > hashconfig->pw_max) plain_len = hashconfig->pw_max;
 
@@ -147,7 +159,7 @@ int process_stdout (hashcat_ctx_t *hashcat_ctx, hc_device_param_t *device_param,
 
       for (u32 il_pos = 0; il_pos < il_cnt; il_pos++)
       {
-        for (int i = 0; i < 8; i++)
+        for (int i = 0; i < 64; i++)
         {
           plain_buf[i] = pw.i[i];
         }
@@ -217,7 +229,7 @@ int process_stdout (hashcat_ctx_t *hashcat_ctx, hc_device_param_t *device_param,
 
       for (u32 il_pos = 0; il_pos < il_cnt; il_pos++)
       {
-        for (int i = 0; i < 8; i++)
+        for (int i = 0; i < 64; i++)
         {
           plain_buf[i] = pw.i[i];
         }
@@ -254,23 +266,21 @@ int process_stdout (hashcat_ctx_t *hashcat_ctx, hc_device_param_t *device_param,
 
       for (u32 il_pos = 0; il_pos < il_cnt; il_pos++)
       {
-        for (int i = 0; i < 8; i++)
+        for (int i = 0; i < 64; i++)
         {
           plain_buf[i] = pw.i[i];
         }
 
         plain_len = pw.pw_len;
 
-        u64 off = device_param->kernel_params_mp_buf64[3] + il_pos;
+        char *comb_buf = (char *) device_param->combs_buf[il_pos].i;
+        u32   comb_len =          device_param->combs_buf[il_pos].pw_len;
 
-        u32 start = 0;
-        u32 stop  = device_param->kernel_params_mp_buf32[4];
+        memcpy (plain_ptr + plain_len, comb_buf, comb_len);
 
-        memmove (plain_ptr + stop, plain_ptr, plain_len);
+        plain_len += comb_len;
 
-        sp_exec (off, (char *) plain_ptr, mask_ctx->root_css_buf, mask_ctx->markov_css_buf, start, start + stop);
-
-        plain_len += start + stop;
+        if (plain_len > hashconfig->pw_max) plain_len = hashconfig->pw_max;
 
         out_push (&out, plain_ptr, plain_len);
       }
