@@ -27,7 +27,7 @@ inline double hc_timer_get (hc_timer_t a)
   return (double) ((double) (hr_tmp.QuadPart - a.QuadPart) / (double) (hr_freq.QuadPart / 1000));
 }
 
-#elif defined(_POSIX)
+#else
 
 inline void hc_timer_set (hc_timer_t* a)
 {
